@@ -1,29 +1,40 @@
-import io
 import socket
 import struct
 
+import six
+from six import byte2int
+
 
 def unpack_ipv4(data):
-    return socket.inet_ntoa(data)
+    return socket.inet_ntoa(six.b(data))
 
 
 def _get_pairs(data):
-    stream = io.BytesIO(data)
-    while True:
-        b = stream.read(2)
-        if not b:
-            return
-        yield b
+    """Return data in pairs
+
+    This uses a clever hack, based on the fact that zip will consume
+    items from the same iterator, for each reference found in each
+    row.
+
+    Example::
+
+        >>> _get_pairs([1, 2, 3, 4])
+        [(1, 2), (3, 4)]
+
+    """
+    return list(zip(*((iter(data),)*2)))
 
 
 def unpack_ipv6(data):
+    data = six.b(data)
     return ':'.join(
-        '{0:02x}{1:02x}'.format(ord(x), ord(y))
-        for (x, y) in _get_pairs(data))
+        '{0:02x}{1:02x}'.format(x, y)
+        for (x, y) in _get_pairs(six.iterbytes(data)))
 
 
 def unpack_macaddr(data):
-    return ':'.join(format(ord(x), '02x') for x in data)
+    data = six.b(data)
+    return ':'.join(format(x, '02x') for x in six.iterbytes(data))
 
 
 def unpack_euiaddr(data):
@@ -39,7 +50,7 @@ def unpack_timestamp_resolution(data):
     """
     if len(data) != 1:
         raise ValueError('Data must be exactly one byte')
-    num = ord(data)
+    num = byte2int(data)
     base = 2 if (num >> 7 & 1) else 10
     exponent = num & 0b01111111
     return base ** (-exponent)
